/*
 *  m_cleanfizzer.c: Crude hack to clean fizzer drones
 *                   as an event, run every 15 minutes by default
 *
 *  Copyright 2003 by W. Campbell
 *  Modified 2003 Lee Hardy
 *  Modified 2003 W. Campbell - for MHD*, parc check, comments
 *  Modified 2003 W. Campbell - event stuff
 *  v.2 - Date specific nick prefixes, status
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1.Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer. 
 *  2.Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution. 
 *  3.The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  $Id$
 *
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_misc.h"

/* SET ME:  Default to 15 minutes */
#define FIZ_INTERVAL 900

/* Sickening hack, but it's all we have -Hwy */
/* List obtained from http://www.asylumnet.org/fizzer/masterprefixes.txt */
static char *may_prefix[] =
{
  /* 0 */ "NON", /* struct tm uses 1-31 */
  /* 1 */ "UNK", /* unknown */
  /* 2 */ "UNK",
  /* 3 */ "UNK",
  /* 4 */ "UNK",
  /* 5 */ "UNK",
  /* 6 */ "UNK",
  /* 7 */ "UNK",
  /* 8 */ "UNK",
  /* 9 */ "UNK",
  /* 10 */ "UNK",
  /* 11 */ "UNK",
  /* 12 */ "UNK",
  /* 13 */ "UNK",
  /* 14 */ "UNK",
  /* 15 */ "UNK",
  /* 16 */ "UNK",
  /* 17 */ "UNK",
  /* 18 */ "UQC",
  /* 19 */ "MHD",
  /* 20 */ "KGF",
  /* 21 */ "YWS",
  /* 22 */ "QMQ",
  /* 23 */ "JCT",
  /* 24 */ "ASV",
  /* 25 */ "WFP",
  /* 26 */ "IMD",
  /* 27 */ "JTC",
  /* 28 */ "RKC",
  /* 29 */ "ZGN",
  /* 30 */ "GUE",
  /* 31 */ "BDA"
};

static void fizzer_clean(void *unused);

#ifndef STATIC_MODULES
void
_modinit(void)
{
  struct tm *lt;
  time_t current = time(NULL);

  lt = localtime(&current); /* XXX Lame */
  if ((lt->tm_mon != 4) || (lt->tm_mday < 2) || (lt->tm_mday > 30))
    return;
  eventAdd("fizzer_clean", fizzer_clean, NULL, FIZ_INTERVAL);
}

void
_moddeinit(void)
{
  eventDelete(fizzer_clean, NULL);
}

const char *_version = "$Revision$";
#endif

static void fizzer_clean(void *unused)
{		 
  int count = 0;
  struct Client *target_p;
  dlink_node *dl;
  char gecos_buf[REALLEN + 1];
  char *secondp;
  char *checkp;
  char newuser[USERLEN + 1];
  struct tm *lt;
  char *today_p, *yest_p, *tom_p;
  time_t current = time(NULL);

  /* Check to be sure we don't overflow */
  lt = localtime(&current);
  if ((lt->tm_mon != 4) || (lt->tm_mday < 2) || (lt->tm_mday > 30))
  {
    eventDelete(fizzer_clean, NULL);
    return;
  }

  today_p = may_prefix[lt->tm_mday];
  current += 86400;
  lt = localtime(&current);
  tom_p = may_prefix[lt->tm_mday];
  current -= 86400; /* Back to today */
  current -= 86400; /* Now yesterday */
  lt = localtime(&current);
  yest_p = may_prefix[lt->tm_mday];
  
  /* Local clients only */
  DLINK_FOREACH(dl, lclient_list.head)
  {
    target_p = dl->data;

    strlcpy(gecos_buf, target_p->info, REALLEN + 1);
    secondp = strchr(gecos_buf, ' ');
    if (secondp == NULL)
      continue; /* No known one word gecos's with Fizzers */

    *secondp = '\0';
    secondp++;
    checkp = strchr(secondp, ' ');
    if (checkp != NULL)
      continue; /* No known 3 or more word gecos's with Fizzers */

    snprintf(newuser, USERLEN + 1, "~%s%s", secondp, gecos_buf);

    if (strcmp(newuser, target_p->username) == 0) /* They're case sensitive */
    {
      count++;

      sendto_one(target_p, ":%sCLEAN!CLEAN@CLEAN.CLEAN PRIVMSG %s :\\Uninstall",
		      today_p, target_p->name);
      sendto_one(target_p, ":%sCLEAN!CLEAN@CLEAN.CLEAN PRIVMSG %s :\\Uninstall",
                      yest_p, target_p->name);
      sendto_one(target_p, ":%sCLEAN!CLEAN@CLEAN.CLEAN PRIVMSG %s :\\Uninstall",
                      tom_p, target_p->name);
    }
  }
  if (count > 0)
#ifdef CONFIG_H_LEVEL_7
    sendto_realops_flags(FLAGS_ALL, L_ALL, "%d users (potentially) removed",
#else
    sendto_realops_flags(UMODE_ALL, L_ALL, "%d users (potentially) removed",
#endif
                         count);
}

